#include "Game.h"

//App name for windows, errors, and messages
const char app_name[]="StarFighter 3D";
const int app_width=750; //define app width and height
const int app_height=600;
bool app_done=false; // On true, clean up and exit app
bool lighting_on=true;//Light on or off?
D3DXVECTOR3 pos(0.0f, 0.0f, -15.0f); //this is the position of the "camera"
D3DXVECTOR3 lookat(0.0f, 0.0f, 0.0f);//this is where we are looking towards

// D3DDevice is the main rendering interface. When you do anything to be displayed on 
//screen, chances are it will be done through this device. 
IDirect3DDevice8 *app_d3d_device=NULL; 

Game::Game() //we give values to the variables that need them before the actual game starts
{
	score = 0;
	contNum = 600;
	lastUpdate = 0;
	fpsUpdateInterval = 0.5f;
	numFrames = 0;
	fps = 0;
	app_main_window=NULL;
	app_D3D=NULL;
}

//The FATALERROR function goes through the program end process if we encounter
//an error that will shut down the program
void Game::FatalError(const char *error_msg)
{
   kill_scene();
   kill_d3d();
   kill_window();
   OutputDebugString( error_msg );
   OutputDebugString("\n");
   MessageBox(NULL, error_msg,app_name, MB_OK );
   exit(5);
}

//the FIND_16BIT_MODE will find the proper mode for use on a computer. The majority of
//computers will support one of these two modes.
D3DFORMAT Game::find_16bit_mode(void)
{
	HRESULT hr;

   //Test for R5G6B5.  All 16-bits are used, full 64K of color
   hr=app_D3D->CheckDeviceType(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,D3DFMT_R5G6B5,D3DFMT_R5G6B5,FALSE);
   if(SUCCEEDED(hr)){
      OutputDebugString("D3DFMT_R5G6B5\n");
      return D3DFMT_R5G6B5;
   }

   //Test X1R5G5B5. Technically 15-bit and only 32K colours
   hr=app_D3D->CheckDeviceType(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,D3DFMT_X1R5G5B5,D3DFMT_X1R5G5B5,FALSE);
   if(SUCCEEDED(hr)){
      OutputDebugString("D3DFMT_X1R5G5B5\n");
      return D3DFMT_X1R5G5B5;
   }

   //Graphics card doesn't support either for some reason
   FatalError("Couldn't find a supported mode\n");

   return (D3DFORMAT)NULL;
}

//MESSAGE_PUMP does what it sounds like, takes waiting message and displays them
void Game::message_pump(void)
{
	MSG msg;
   if(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE)){
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
}

//INIT_D3D is a one time preperation of certain direct3d objects
void Game::init_d3d(void)
{
	HRESULT hr;
D3DPRESENT_PARAMETERS d3dpp;
D3DDISPLAYMODE display_mode;

   //In a D3D8 program you must create D3D8 first. 
   app_D3D = Direct3DCreate8( D3D_SDK_VERSION );
   if(!app_D3D ){
	   FatalError("Error Creating Direct3D");
   }

   //Get the (desktop) display mode. Only needed in window, not full screen
   hr=app_D3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT,&display_mode);
   if(FAILED(hr)){
      FatalError("Error while getting display mode\n");
   }

   //Clear out D3DPRESENT_PARAMETERS struct. Good idea to zero it out.
   ZeroMemory(&d3dpp,sizeof(d3dpp));

   //Same for full and windowed
   d3dpp.SwapEffect     = D3DSWAPEFFECT_DISCARD; // Get rid of previous frames
   d3dpp.hDeviceWindow  = app_main_window;  //This is our only/main (if more than one) window
   d3dpp.BackBufferCount= 1;  //Only 1 back buffer is needed

   //Z-buffer is needed for everything to render right
   d3dpp.EnableAutoDepthStencil = TRUE;
   d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

   // BackBufferWidth and Height have to be set for full screen
   // these values are to determine the display mode.
   d3dpp.Windowed          = TRUE;
   d3dpp.BackBufferFormat  = display_mode.Format; //same format as desktop, GetAdapterDisplayMode()


   hr=app_D3D->CreateDevice(D3DADAPTER_DEFAULT, //The default adapter, 2+ monitor can have > 1
                            D3DDEVTYPE_HAL, //Hardware acceleration instead of software renderer
                            app_main_window, // This is our window
                            D3DCREATE_SOFTWARE_VERTEXPROCESSING, //Processes vertices in software, 
										//slower than hard but works on all graphics cards
                            &d3dpp, //D3DPRESENT_PARAMETERS struct, tells what to build
                            &app_d3d_device); //will point to new device
   if(FAILED(hr)){
      FatalError("Error creating device\n");
   }

   mShip.Load(app_d3d_device, "Data/ship.x"); //here we actually load of .x files into the 
   mEnemy.Load(app_d3d_device, "Data/enemy1.x"); //created objects
   mBullet.Load(app_d3d_device, "Data/bullet.x");
   fFont.load(app_d3d_device); //load our font
   mSprite.load(app_d3d_device); //load our sprite for later 2d use
   introTexture.load(app_d3d_device, "Data/intro.jpg", 0xFF92260D); //load our textures 
   playTexture.load(app_d3d_device, "Data/back.jpg", 0xFFFFFFFF);  
   D3DXIMAGE_INFO tempInfo = playTexture.getInfo(); //use a tempinfo for easier use 
   playTexture.setPos1(0, (app_height-(float)tempInfo.Height)); //we set these positions
   D3DXVECTOR2 tempVector = playTexture.getPos1();     //so we can have proper scrolling
   playTexture.setPos2(0, -((float)tempInfo.Height - tempVector.y));
   livesTexture.load(app_d3d_device, "Data/lives.jpg", 0xFFFFFFFF);
   livesTexture.setPos1(625, 70);
   kInput.load(app_main_window); //load our keyboard device
   gState = INTRO; //lastly, tell the program we should be at the INTRO of the game
}

//INIT_SCENE is also a one time preperation of direct3d objects
void Game::init_scene(void)
{
	D3DXMATRIX view_matrix;
	D3DXMATRIX matProj;	
	canShoot = 0;
	player.changePos(0, -5);
	player.bSphere.setRadius(mShip.getRadius()*.5);

   //D3DTSS_COLOROP is how the color of pixels is determined
   //D3DTOP_SELECTARG1 looks at D3DTSS_COLORARG1 to determine the color.
   //MODULATE determines color by COLORARG1 and COLORARG2
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLOROP, D3DTOP_MODULATE);

   //D3DTSS_COLORARG1 is set to D3DTA_TEXTURE, taking the color completly from the 
   //texture, we could use D3DTA_DIFFUSE to use the color
   //CURRENT is vertex color if light off, or result of lighting if it is on
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLORARG1, D3DTA_TEXTURE);
   app_d3d_device->SetTextureStageState(0,D3DTSS_COLORARG2, D3DTA_CURRENT);

   //Turn off alphablending/transparency to make it faster
   app_d3d_device->SetTextureStageState(0,D3DTSS_ALPHAOP, D3DTOP_DISABLE);
   //turn off culling to see room everywhere
   app_d3d_device->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
   //MAGFILTER is how the texture is drawn when it is bigger than the real size
   //MINFILTER is when it is drawn smaller. 
   //D3DTEXF_LINEAR & D3DTEXF_POINT. Linear is smoother but more expensive.
   app_d3d_device->SetTextureStageState(0,D3DTSS_MAGFILTER, D3DTEXF_NONE);
   app_d3d_device->SetTextureStageState(0,D3DTSS_MINFILTER, D3DTEXF_NONE);

   //Zero the light struct and fill in the fields needed
   ZeroMemory( &light, sizeof(D3DLIGHT8) );
   light.Type=D3DLIGHT_POINT;//Point light, sinces in all directions
   light.Diffuse.r=2.0f; //Light color
   light.Diffuse.g=2.0f;
   light.Diffuse.b=2.0f;
   light.Range=1000.0f;//Range for illumination
   light.Position=D3DXVECTOR3(0.0f,2.0f,-10.0f); //position of the light
   light.Attenuation0=1.0f;//How light falls off, set to full illumination, no fall off
   
   app_d3d_device->SetLight( 0, &light );//set our light to an index
   app_d3d_device->LightEnable( 0, TRUE); //Enable our light, however this doesn't mean the light is on.

   //Turn off D3D lighting //ambient is default lighting for objects not hit by our light
   app_d3d_device->SetRenderState( D3DRS_LIGHTING, TRUE );
   app_d3d_device->SetRenderState(D3DRS_AMBIENT,0x00050505);

   // Turn on the zbuffer.  Zbuffer draws things in the proper order (overlapping items
   // appear how they should, whatever is in front you see.)
   app_d3d_device->SetRenderState( D3DRS_ZENABLE, TRUE );

   //Build View Matrix...(camera), viewpoint, looking to, what is up
   D3DXMatrixLookAtLH(&view_matrix,&pos,
                                   &lookat,
                                   &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));

   //We set the camera, but will be altering it later
   app_d3d_device->SetTransform(D3DTS_VIEW,&view_matrix);

   D3DXMatrixPerspectiveFovLH(&matProj, //Resulting Matrix
                              D3DX_PI/4,//Field of View, in radians. (PI/4 = typical)
                              ((float)app_width / (float)app_height), //Aspect ratio
                              1.0f,      //Near view plane
                              100.0f ); // Far view plane

   //Projection matrix
   app_d3d_device->SetTransform( D3DTS_PROJECTION, &matProj );
}

//in RENDER is where we draw everything to the surface. We can also make translations
//or rotations.
void Game::render(void)
{
	D3DXMATRIX matWorld;
	D3DXMATRIX view_matrix;

	//set our matrix that we are looking at. this takes the position, look at, and up coords
   D3DXMatrixLookAtLH(&view_matrix,
                         &pos,
                         &lookat,
                         &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ));

   app_d3d_device->SetTransform(D3DTS_VIEW,&view_matrix);

   //clear buffer to color and clear zbuffer
   app_d3d_device->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER , D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );

   //tell device we are ready to draw/render
   if(SUCCEEDED(app_d3d_device->BeginScene())){
		//determine what to draw based on gamestate
	   if(gState == INTRO) //if intro we draw the background and the "blinking" text
	   {
		   intro();
	   } //if we are not at the intro we always want the screen to scroll so run this
	   else if (gState != INTRO) //also we still want objects drawn on screen
	   {
		   playTexture.scroll(); //scroll the background
		   mSprite.start(); //tell the sprite we are ready to use it
		   //based on values, draw textures to the sprite therefore if the texture is off
		   //screen we do not have to draw it
		   if(playTexture.getPos1Y()>-1000) 
		   mSprite.draw(playTexture.getTexture(), playTexture.getScaling(), playTexture.getPos1());
		   if(playTexture.getPos2Y()>-1000)
			   mSprite.draw(playTexture.getTexture(), playTexture.getScaling(), playTexture.getPos2());
		   if(playTexture.getPos1Y()>600)
                   playTexture.setPos1(0, -1400);
		   if(playTexture.getPos2Y()>600)
                   playTexture.setPos2(0, -1400);
		   mSprite.end(); //tell the sprite we are done. We do this outside of the draw
		   //because it takes up less process time rather than doing it for EACH draw

		   char buffer[20]; //create local variables for the text to draw on screen
		   char health[20];
		   char scores[20];
		   sprintf(buffer, "FPS: %f", fps); //insert the text to draw
		   sprintf(health, "Health: %d", player.getHealth());
		   sprintf(scores, "Score: %d", score);
		   fFont.Draw(buffer, 200, 20, D3DCOLOR_XRGB (255, 50, 50)); //draw the strings
		   fFont.Draw(health, 1360, 20, D3DCOLOR_XRGB (50, 50, 255));
		   fFont.Draw(scores, 1360, 40, D3DCOLOR_XRGB (50, 50, 255));

		   for(int i = 0; i<player.getLives(); i++)
		   {
			   D3DXVECTOR2 tempPos = livesTexture.getPos1();
			   tempPos.x = tempPos.x + (i * 40);
			   mSprite.draw(livesTexture.getTexture(), livesTexture.getScaling(), tempPos);
		   }

		   if(!player.isDead()) //we only draw the player if he is alive
			   mShip.draw(app_d3d_device, player.getPositionVec());
		   
		   //set the iterator to the beginning of the enemy list so we can traverse it
		   //and draw the enemies at the correct positions. We do the same for the player
		   //bullers and enemy bullets.
		   eIter = enemies.begin(); 
		   for (int j = 0; j<enemies.size(); j++)
		   {
			   mEnemy.draw(app_d3d_device, eIter->getPositionVec());
			   eIter++;
		   }

		   bIter = bullets.begin();
		   for (int j = 0; j<bullets.size(); j++)
		   {
			   mBullet.draw(app_d3d_device, bIter->pos);
			   bIter++;
		   }

		   eBIter = eBullets.begin();
		   for (int j = 0; j<eBullets.size(); j++)
		   {
			   mBullet.draw(app_d3d_device, eBIter->pos);
			   eBIter++;
		   }

		   //the following will only happen when a game is over. put up a continue 
		   //countdown with instructions on what actions to take. if the countdown reaches
		   //0 we reset the game
		   if(gState==OVER)
		   {
			   contNum--;
			   Font contFont;
			   contFont.load(app_d3d_device);
			   char cont[50];
			   sprintf(cont, "Continue = Enter~~~Exit = Esc");
			   char contCount[20];
			   sprintf(contCount, "%d", contNum);
			   contFont.Draw(cont, 615, 300, D3DCOLOR_XRGB(255,20,20));
			   contFont.Draw(contCount, 640, 320, D3DCOLOR_XRGB(255,20,20));
			   if(contNum==0)
			   {
				   contNum=600;
				   reset();
			   }
		   }

		   //set the world matrix
		   app_d3d_device->SetTransform(D3DTS_WORLD,&matWorld );
	   }

   //Tell device this frame is done being rendered
   app_d3d_device->EndScene();
   }
   //Display what we did
   app_d3d_device->Present( NULL, NULL, NULL, NULL );
   //good idea to set texture to NULL so your textures can be released properly
   app_d3d_device->SetTexture(0,NULL);
}

//INIT_WINDOW is pretty self explanatory. We create our window with the proper variables
//and style
void Game::init_window(void)
{
	ULONG window_width, window_height;
	WNDCLASS window_class;
	DWORD style;

   //WNDCLASS structure.  Window classes = template for win creation
   //can have many windows from one class
   window_class.style          = CS_OWNDC;
   window_class.cbClsExtra     = 0;
   window_class.cbWndExtra     = 0;
   window_class.hInstance      = app_instance;
   window_class.hIcon          = LoadIcon(NULL,IDI_APPLICATION);
   window_class.hCursor        = LoadCursor(NULL,IDC_ARROW);
   window_class.hbrBackground  = (HBRUSH)GetStockObject(BLACK_BRUSH);
   window_class.lpszMenuName   = NULL;
   window_class.lpszClassName  = "DH Class";
   //default window handler, all windows messages sent to this function.
   window_class.lpfnWndProc    = default_window_proc;

   //Register the class
   if(!RegisterClass(&window_class)){
      FatalError("Error registering window class");
   }

   window_width=app_width; //Since we are not doing full screen, we always have the width
   window_height=app_height; //and height
   style=WS_OVERLAPPED|WS_SYSMENU;
 
   //create our window with the class, name, style, width and height and the app instance
   app_main_window=CreateWindow("DH Class",app_name,style,
                              0,0,window_width,window_height,NULL,NULL,
                              app_instance,NULL);

   if(!app_main_window){
      FatalError("Error creating window");
   }

   //Makes sure the window is visible and has focus
   ShowWindow(app_main_window,SW_SHOW);
   UpdateWindow(app_main_window);
   SetFocus(app_main_window);
}

//KILL_D3D goes through and releases everyything that pertains to d3d in the opposite order
//of creation
void Game::kill_d3d(void){

   if(app_d3d_device){
      app_d3d_device->Release();
      app_d3d_device=NULL;
   }

   if(app_D3D){
      app_D3D->Release();
      app_D3D=NULL;
   }
}

//KILL_SCENE can be used to release any non d3d related objects
void Game::kill_scene(void)
{}

//KILL_WINDOW will release/unregister the window
void Game::kill_window(void){
  
   if(app_main_window){ //is the window valid
      if(!DestroyWindow(app_main_window)){ //if destroying window fails
         OutputDebugString("Failed to DestroyWindow\n");
         MessageBox(NULL,"Destroy Window Failed",app_name,MB_OK|MB_ICONERROR|MB_TOPMOST);
      }else{
         MSG msg;
         while(PeekMessage(&msg, NULL, 0, 0,PM_REMOVE)){ //clean pending messages
            DispatchMessage(&msg);
         }
      }
      app_main_window=NULL; //set window handle to NULL, to be safe
   }

   //Unregister the window. If you have more than one window with the class
   //you need to close them all before unregistering
   if(!UnregisterClass("DH Class",app_instance)){
      OutputDebugString("Failed to Unregister Window\n");
      MessageBox(NULL,"Unregister Failed",app_name,MB_OK|MB_ICONERROR|MB_TOPMOST);
   }
}

//this is a function that can be used for keyboard input, but since we are using directinput
//it is only used for closing the app without key input, telling windows the app is being
//destroyed, and repainting the window.
LRESULT CALLBACK Game::default_window_proc(HWND hwnd,UINT msg,WPARAM wparam,LPARAM lparam){
HDC hdc;
PAINTSTRUCT ps;
   
int virt_key = (int) wparam;
   switch(msg){
      case WM_PAINT:    //Repaint the window
         hdc=BeginPaint(hwnd,&ps);
         EndPaint(hwnd,&ps);
         return 0;
      case WM_CLOSE:    // If close button is hit, end the application
         app_done=true;
         return 0;
      case WM_DESTROY:  //If the window is being destroyed, tell Windows
         PostQuitMessage(0);
         return 0;
   }
   return (DefWindowProc(hwnd,msg,wparam,lparam));
}

//RUN is our main function that cycles through the application and runs the other
//functions that are needed on a per frame basis
void Game::run(void)
{
	InitGameTime(); //get the time at the start of game, used for FPS
   init_window();// Build our window to screen size choice

   init_d3d();//Build the D3D objects we need   
   init_scene();//Once prep of things required for rendering
   srand(time(NULL)); //this is used for random numbers, seed with time

   //This will loop until the app is terminated
   while(!app_done){
      message_pump(); //Check for window messages
	  keyboard(); //get keyboard input
	  UpdateFPS();
	  if(gState != INTRO)
		  gameLogic(); //check for boundaries, collisions, deaths, etc.
      render(); //draw!
   }
   kill_scene(); 
   kill_d3d(); //clean D3D objects
   kill_window(); //...closes the window...
}

//What we do here is get the keyboard information for the frame and see if it has
//an effect on the program
void Game::keyboard(void)
{
	char     buffer[256]; 
	HRESULT  hr; 

	//get the current state of the keyboard to see if a key is pressed
    hr = kInput.dKeyboard->GetDeviceState(sizeof(buffer),(LPVOID)&buffer); 
	if FAILED(hr) 
	{ 
		return; 
	} 

	//the following are pretty easy to understand. If direction, move that direction
	if (gState == PLAYING)
	{
		if (KEYDOWN(buffer, DIK_RIGHT) && player.getPositionX() < 4.55)
			player.changePos(.08, 0);
		if(KEYDOWN(buffer, DIK_LEFT) && player.getPositionX() > -7.2) 
			player.changePos(-.08, 0);
		if (KEYDOWN(buffer, DIK_UP) && player.getPositionY() < 5.5)
			player.changePos(0, .08);
		if (KEYDOWN(buffer, DIK_DOWN) && player.getPositionY() > -5.5)
			player.changePos(0, -.08);
		if (KEYDOWN(buffer, DIK_SPACE) && canShoot == 0) //if SPACE is pushed we create a
		{												//new bullet and give it info
			bullets.push_back(*new Bullet);
			bullets.back().pos = player.getPositionVec();
			bullets.back().bSphere.setCenter(player.getPositionVec());
			bullets.back().bSphere.setRadius(mBullet.getRadius()*.7);
			canShoot++;
		}
	}
	if (KEYDOWN(buffer, DIK_RETURN) && gState == INTRO) //ENTER is pushed to start the game
		gState = PLAYING;
	if (KEYDOWN(buffer, DIK_RETURN) && gState == OVER) //again, ENTER to start,but when dead
		gameContinue();
	if (KEYDOWN(buffer, DIK_ESCAPE)) //kill the program
		app_done = true;
}

//GAMELOGIC cycles through the existing objects and alters them to how they are going to be
//for the next frame. Some of these only happen when playing such as new enemies
void Game::gameLogic()
{
	player.bSphere.setCenter(player.getPositionVec()); //make sure our sphere is where the
														//player is
	if(gState==PLAYING) //only if we are playing should we create new enemies
	{
		if(rand()%1000>993 && enemies.size()<5)
		{
		enemies.push_back(*new Ships());
		enemies.back().changePos(rand()%5, 10); //we give them a semi random x value
		if(rand()%2 == 1) //then possibly change it to the other side of the screen
			enemies.back().changePos(-(enemies.back().getPositionX()*2), 0);
		enemies.back().bSphere.setCenter(enemies.back().getPositionVec());
		enemies.back().bSphere.setRadius(mEnemy.getRadius()*.55); //set tto mesh radius
		}
	}
	

	eIter = enemies.begin(); //cycle through enemies and see if they will shoot or if 
	for(int i = 0; i<enemies.size(); i++) //they are off the screen
	{
		eIter->changePos(0, -.05); //movement for each enemy
		eIter->bSphere.setCenter(eIter->getPositionVec());
		if(rand()%1000>993)
		{
			eBullets.push_back(*new Bullet);
			eBullets.back().pos = eIter->getPositionVec(); //bullets start where the ship is
			eBullets.back().bSphere.setRadius(mBullet.getRadius()*.7);
		}
		if(eIter->getPositionY() < -8) //delete if off screen
		{
			eTempIter=eIter;
			eTempIter++;
			enemies.erase(eIter);
			eIter=eTempIter;
		}
		else //otherwise go to next in list
			eIter++;
	}
    
	bool intersect;
	eIter = enemies.begin(); //again with the enemies, but this time to determine
	for(int i = 0; i<enemies.size(); i++) //if they collide with player bullets
	{
		intersect = false;
		bIter = bullets.begin();
		for(int j = 0; j<bullets.size(); j++)
			{
				//if they do collide, delete the bullet and set intersect to true 
				//so we know to delete that ship. add 3 to player score
				if(bIter->bSphere.SphereIntersect(&eIter->bSphere)) 
				{                                
					tempIter=bIter;
					tempIter++;
					bullets.erase(bIter);
					bIter=tempIter;
					intersect = true;
					score+=3;
				}
				else
					bIter++;
			}
		if (intersect) //detele the ship if it collided with a bullet
		{
			eTempIter=eIter;
			eTempIter++;
			enemies.erase(eIter);
			eIter=eTempIter;
		}
		else
			eIter++;
	}

	bIter = bullets.begin(); //moves the player bullets and deletes them if off screen
	for(int i = 0; i<bullets.size(); i++)
	{
		bIter->pos.y += .17;
		bIter->bSphere.setCenter(bIter->pos);
		if(bIter->pos.y > 10)
		{
			tempIter=bIter;
			tempIter++;
			bullets.erase(bIter);
			bIter=tempIter;
		}
		else
			bIter++;
	}

	eBIter = eBullets.begin(); //moves enemy bullets and deletes if off sceen or if they
	for(int i = 0; i<eBullets.size(); i++) //collide with the player
	{
		eBIter->pos.y -= .12;
		eBIter->bSphere.setCenter(eBIter->pos);
		//if it DOES NOT collide and the player is alive then check for off screen
		if(!eBIter->bSphere.SphereIntersect(&player.bSphere) && !player.isDead())
		{
			if(eBIter->pos.y < -10)
			{
				eBtempIter=eBIter;
				eBtempIter++;
				eBullets.erase(eBIter);
				eBIter=eBtempIter;
			}
			else
				eBIter++; //only if no collide and on screen
		}
		else //or delete if collide
		{
			eBtempIter=eBIter;
			eBtempIter++;
			eBullets.erase(eBIter);
			eBIter=eBtempIter;
			player.changeHealth(-3);
		}
	}
    
	if(canShoot > 0) //odd thing for determining fire rate
		canShoot++;
	if (canShoot == 25)
		canShoot = 0;

	eIter = enemies.begin(); //see if enemies collide with the player ship
	for (int i = 0; i<enemies.size(); i++)
	{
		if(player.bSphere.SphereIntersect(&eIter->bSphere) && !player.isDead())
		{
			eTempIter=eIter; //if they do detele them and add 5 to score but -25 from health
			eIter++;
			enemies.erase(eTempIter);
			player.changeHealth(-25);
			score+=5;
		}
		else //cycle through the list
			eIter++;
	}

	if (player.getHealth()<=0 && player.getLives() == 0) //if the player has 0 or less health then the game is over
			{
				player.changeHealth(-player.getHealth());
				gState=OVER;
				player.changeDead(true);
			}
	else if (player.getHealth()<=0 && player.getLives() > 0)
	{
		player.changeLives(-1);
		player.resetValues(100, D3DXVECTOR3 (0, -5, 0), player.getLives());
	}
}

void Game::InitGameTime()
{
  // We need to know how often the clock is updated
  if( !QueryPerformanceFrequency((LARGE_INTEGER *)&ticksPerSecond) )
    ticksPerSecond = 1000;

  // If timeAtGameStart is 0 then we get the time since
  // the start of the computer when we call GetGameTime()
  timeAtGameStart = 0;
  timeAtGameStart = GetGameTime();
}

// Called every time you need the current game time
float Game::GetGameTime()
{
  UINT64 ticks;
  float time;

  // This is the number of clock ticks since start
  QueryPerformanceCounter((LARGE_INTEGER *)&ticks); 

  // Divide by frequency to get the time in seconds
  time = (float)(__int64)ticks/(float)(__int64)ticksPerSecond;

  // Subtract the time at game start to get
  // the time since the game started
  time -= timeAtGameStart;

  return time;
}

//if our update interval is less than the passed time, update the FPS
void Game::UpdateFPS()
{
  numFrames++;
  float currentUpdate = GetGameTime();
  if( currentUpdate - lastUpdate > fpsUpdateInterval )
  {
    fps = numFrames / (currentUpdate - lastUpdate);
    lastUpdate = currentUpdate;
    numFrames = 0;
  }
}

//this is what we do during the intro, which consists of drawing background and blinking
//text

void Game::intro(void)
{
	mSprite.start();
	mSprite.draw(introTexture.getTexture(), introTexture.getScaling(), introTexture.getPos1());
	mSprite.end();
	fFont.blink(1);
	if (fFont.blink(0) < 60)
		fFont.Draw("Press Enter", 170, 525, 0xFFCC2222);
	if (fFont.blink(0) == 120)
		fFont.blink(-(fFont.blink(0)));
}
//RESET the values for our game objects. clear out our linked lists so that we start
//fresh with nothing but the background and playe ron screen. set to INTRO
void Game::reset(void)
{
	gState=INTRO;
	player.resetValues(100, D3DXVECTOR3 (0, -4, 0), 3);
	enemies.clear();
	bullets.clear();
	eBullets.clear();
	contNum=600;
}

//only Different from reset is the gamestate is set to playing and you immediately jump
//back into action
void Game::gameContinue(void)
{
	gState=PLAYING;
	player.resetValues(100, D3DXVECTOR3 (0, -4, 0), 3);
	enemies.clear();
	bullets.clear();
	eBullets.clear();
	contNum=600;
}

Game::~Game()
{}